{% if toolbar_search_form %}
    <form class="form-inline">
        <span class="label label-xlg label-primary arrowed-right">层级过滤</span>
        {% for field in toolbar_search_form %}
            {{ field }}
        {% endfor %}
    </form>
    <script>
        {% for field in toolbar_search_form %}
            $('select[name="{{field.name}}"]').change(function () {
                updateHashParam({'{{field.name}}': $(this).val(), 'page': 1})
            });
        {% endfor %}
    </script>

{% endif %}